-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function UnmappedLands2:StartActivity()
	print ("UnmappedLands2:Ally_Test:StartActivity");

	if self.IsInitialized == nil then
		self.IsInitialized = false
	end

	if self.IsInitialized then 
		return
	end

	self.IsInitialized = true
	
	-- Get areas
	self.AtkBrain = SceneMan.Scene:GetArea("AtkBrain")

	-- Get attackers areas
	self.AtkTrooper = {}
	for i = 1, CF_MaxAssaultPresets do
		self.AtkTrooper[i] = SceneMan.Scene:GetArea("AtkTrooper"..i)
	end
	
	self.GS = {};
	self.ModuleName = "UnmappedLands2.rte";
	
	CF_InitFactions(self)
	self:LoadCurrentGameState();
	
	-- In ally missions player is always an attacker
	self.AttackPlayer = 0
	self.AttackPlayerFaction = CF_GetPlayerFaction(self.GS, self.AttackPlayer)

	self.Base = tonumber(self.GS["BaseTerritory"])
	
	-- Spawn player brain
	local brain = CF_MakeBrain(self.GS, self.AttackPlayer, true)
	if brain ~= nil then
		brain.Team = CF_PlayerTeam
		brain.Pos = self.AtkBrain:GetCenterPoint();
		MovableMan:AddActor(brain);
	end

	-- Spawn player attackers
	for i = 1, CF_MaxAssaultPresets do
		local pre = self.GS["PlayerAssaultPreset"..i]
		if pre ~= nil then
			pre = tonumber(pre)
	
			local actor = CF_MakeActorFromPreset(self.GS, self.AttackPlayer, self.Base, pre, true)
			
			if actor ~= nil then
				actor.Pos = self.AtkTrooper[i]:GetCenterPoint()
				actor.Team = CF_PlayerTeam
				MovableMan:AddActor(actor)
			end
		end
	end
	
	-- Simple test objective - kill enemy
	local actor = CreateAHuman("Soldier Light", "Coalition.rte")
	actor.Pos = Vector(1000,500)
	actor.Team = CF_CPUTeam
	MovableMan:AddActor(actor)
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.TickTimer = Timer();
	self.TickTimer:Reset();
	self.TickInterval = CF_TacticalTickInterval;
	self.Time = 1;

	self.IsInitialized = true
	
	self:SetTeamFunds(0 , 0);
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function UnmappedLands2:PauseActivity(pause)
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function UnmappedLands2:EndActivity()
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function UnmappedLands2:LoadCurrentGameState()
	if CF_IsFileExists(self.ModuleName , STATE_CONFIG_FILE) then
		self.GS = CF_ReadConfigFile(self.ModuleName , STATE_CONFIG_FILE);
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function UnmappedLands2:SaveCurrentGameState()
	CF_WriteConfigFile(self.GS , self.ModuleName , STATE_CONFIG_FILE);
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function UnmappedLands2:UpdateActivity()
	-- Just check for intialization flags in update loop to avoid unnecessary function calls during all the mission
	if self.IsInitialized == nil then
		self.IsInitialized = false
	end
	
	if not self.IsInitialized then
		--Init mission if we're still not
		self:StartActivity()
		return;
	end

	self:ClearObjectivePoints();
	
	-- Clear player's money to avoid dumb cheating
	local gold2add = self:GetTeamFunds(CF_PlayerTeam);
	self:SetTeamFunds(0,0);
	if gold2add ~= 0 then
		CF_SetPlayerGold(self.GS, self.HumanPlayer, CF_GetPlayerGold(self.GS, self.HumanPlayer) + gold2add);
	end
	
	-- Tick timer
	if self.TickTimer:IsPastSimMS(self.TickInterval) then
		self.Time = self.Time + 1
		self.TickTimer:Reset();
	end
	
	self:DoBrainSelection()
	self:CheckWinningConditions();
	self:YSortObjectivePoints();
	CF_ReturnOnMissionEnd();--]]--
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function UnmappedLands2:CheckWinningConditions()
	if self.ActivityState ~= Activity.OVER then
		local playerwin = false;
		local playerlose = false;
	
		if self.PlayerBrainDead then
			print ("Player brain dead")
			playerlose = true;
		end

		-- Define wining conditions here
		local count = 0
		
		for actor in MovableMan.Actors do
			if actor.Team == CF_CPUTeam then
				count = count  + 1
			end
		end
		
		if count == 0 then
			print ("All enemy actors are dead")
			playerwin = true
		end

		if playerlose then
			-- Defeat
			self.WinnerTeam = CF_CPUTeam;
			CF_CommitMissionResult(self.GS, "Defeat")
			ActivityMan:EndActivity();
			self:SaveCurrentGameState();
		end

		if playerwin then
			-- Victory
			self.WinnerTeam = CF_PlayerTeam;
			CF_CommitMissionResult(self.GS, "Victory")
			
			-- Add diplomacy points
			local toadd = tonumber(self.GS["AllyMissionReward"])
			toadd = toadd + (toadd * CF_RelationsBonuses[CF_GetPlayerFaction(self.GS, 0)] / 100)

			toadd = math.ceil(toadd)
			
			print ("Ally reward: "..toadd)
			
			CF_SetPlayerRelations(self.GS, 0, CF_GetPlayerRelations(self.GS, 0) + toadd);
			
			self.GS["LastMissionMessage"] = toadd .. " diplomacy points received."
			
			ActivityMan:EndActivity();
			self:SaveCurrentGameState();
		end
	end
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function UnmappedLands2:DoBrainSelection()
	if self.ActivityState ~= Activity.OVER then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);
			local brain = self:GetPlayerBrain(player);

			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				if team == CF_PlayerTeam then
					self.PlayerBrainDead = true
				end

				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
					
					if team == CF_PlayerTeam then
						self.PlayerBrainDead = false
					end
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Thats all folks!!!
-----------------------------------------------------------------------------------------