-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function WeaponTest:StartActivity()
	print ("WeaponTest:Tactics:StartActivity");

	if self.IsInitialized == nil then
		self.IsInitialized = false
	end

	if self.IsInitialized then 
		return
	end
	
	------ USER SETTINGS ------------------------------
	self.RestartEvery = 10 -- Restart activity every N rounds (will capture cursor after every restart) (all selections will be lost!!!) Set to 0 to disable.
	self.Randomize = true -- Enable random faction selection for each round, if PrimaryFaction is set only 2nd faction will be random
	self.BudgetBased = false -- Turn budget mode on/off. In budget mode every squad costs 1500 oz of gold, if turned off squads will always have exactly 8 units. Stats for every mode are saved separately.
	self.PrimaryFaction = nil
	--self.PrimaryFaction = "S.H.I.E.L.D." -- Set fixed primary faction to test, set to nil to disable
	self.TestedAgainstFactions = nil
	-- Set faction list to test against instead of fully random. For example to test against vanilla factions only
	--self.TestedAgainstFactions = {"Coalition", "Ronin", "Dummy", "Techion", "Imperatus" , "Browncoats"}
	---------------------------------------------------
	
	self.ModuleName = "WeaponTest.rte";
	
	LIB_PATH = self.ModuleName.."/Scripts/"
	BASE_PATH = self.ModuleName.."/Scripts/"

	dofile(LIB_PATH.."CF_ConfigLibrary.lua");
	dofile(LIB_PATH.."CF_MissionLibrary.lua");	
	
	self.IsInitialized = true
	
	CF_InitFactions(self)
	self.PlayableFactions = {}
	for i = 1, #CF_Factions do
		if CF_FactionPlayable[CF_Factions[i]] then
			self.PlayableFactions[#self.PlayableFactions + 1] = CF_Factions[i]
		end
	end
	
	self.LaunchCount = 0
	
	self.EnemySpawnInterval = 25
	self.Budget = 1500
	self.SpawnAmount = 8
	
	self.BudgetBasedStart = self.BudgetBased
	
	if self.PrimaryFaction == nil then
		self.Faction1 = math.random(#self.PlayableFactions)
		self.StrFaction1 = self.PlayableFactions[ self.Faction1 ]
	else
		self.StrFaction1 = self.PrimaryFaction
	end

	local ok = false
	
	while not ok do
		if self.TestedAgainstFactions == nil then
			self.Faction2 = math.random(#self.PlayableFactions)
			self.StrFaction2 = self.PlayableFactions[ self.Faction2 ]
		else
			--self.Faction2 = math.random(#self.PlayableFactions)
			self.StrFaction2 = self.TestedAgainstFactions[math.random(#self.TestedAgainstFactions)]
		end
		
		if self.StrFaction2 ~= self.StrFaction1 then
			ok = true
		end
	end
	
	self.Data = CF_ReadConfigFile(self.ModuleName, "Stats.txt")
	
	self.SelectedValue = 1
	
	-- Get areas
	self.Controller = SceneMan.Scene:GetArea("Controller")

	self.Spawn = {}
	self.Spawn[0] = SceneMan.Scene:GetArea("Spawn1")
	self.Spawn[1] = SceneMan.Scene:GetArea("Spawn2")
	self.Destination = SceneMan.Scene:GetArea("Destination")
	
	self.RecordID = nil
	
	self.PlayerTeam = 0

	-- Spawn controller
	local actor = CreateActor("Controller", self.ModuleName)
	if actor ~= nil then
		actor.Team = self.PlayerTeam
		actor.Pos = self.Controller:GetCenterPoint();
		MovableMan:AddActor(actor);
		
		self:SwitchToActor(actor, 0, 0);
	end

	local actor = CreateActor("Observer", self.ModuleName)
	if actor ~= nil then
		actor.Team = self.PlayerTeam
		actor.Pos = self.Controller:GetCenterPoint() + Vector(0,-100);
		MovableMan:AddActor(actor);
		
		self:SwitchToActor(actor, 0, 0);
	end
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();
	
	self.IsInitialized = true
	
	self.SpawnNextWave = true
	
	self:SetTeamFunds(0, 0);
	
	self:DumpStatsToCSV()
end
-----------------------------------------------------------------------------------------
-- Make brain where c - config, p - player, pay - if true then pay for unit
-----------------------------------------------------------------------------------------
function WeaponTest:MakeActor(preset, module)
	local actor = nil
	local weapon = nil;
	
	actor = CreateAHuman(preset, module)
	if actor ~= nil then
		actor.AIMode = Actor.AIMODE_SENTRY;
	end
	
	return actor;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function WeaponTest:SpawnRandomInfantry(team , pos , faction , aimode)
	--print ("CF_SpawnRandomInfantry");
	local actor = nil;
	local r1, r2;
	local item;
	
	local price = 0
	
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		-- Find AHuman
		local ok = false;
		-- Emergency counter in case we don't have AHumans in factions
		local counter = 0
		
		while (not ok) do
			ok = false;
			r1 = math.random(#CF_ActNames[faction])

			if CF_ActTypes[faction][r1] ~= CF_ActorTypes.TURRET and CF_ActPowers[faction][r1] > 0 and CF_ActClasses[faction][r1] ~= "ACrab" then
				ok = true;
			end
			
			-- Break to avoid endless loop
			counter = counter + 1
			if counter > 80 then
				break
			end
		end
		
		local class = CF_ActClasses[faction][r1]
		
		if class == nil then
			class = "AHuman"
		end
		
		price = price + CF_ActPrices[faction][r1]

		if class == "AHuman" then
			local giveweapons = true

			if CF_PreEquippedActors[faction] ~= nil then
				if CF_PreEquippedActors[faction] == true then
					giveweapons = false
				end
			end
		
			if giveweapons then
				-- Find suitable weapon
				local ok = false;
				-- Emergency counter in case we don't have AHumans in factions
				local counter = 0
				
				while (not ok) do
					ok = false;
					r2 = math.random(#CF_ItmNames[faction])
			
					if (CF_ItmTypes[faction][r2] == CF_WeaponTypes.RIFLE or CF_ItmTypes[faction][r2] == CF_WeaponTypes.SHOTGUN or 
						CF_ItmTypes[faction][r2] == CF_WeaponTypes.SNIPER or CF_ItmTypes[faction][r2] == CF_WeaponTypes.HEAVY) and CF_ItmPowers[faction][r2] > 0 then
						ok = true;
					end
					
					-- Break to avoid endless loop
					counter = counter + 1
					if counter > 40 then
						break
					end
				end
				
				price = price + CF_ItmPrices[faction][r2]
			end
		end
		
		local ok2spawn = true
		
		if self.BudgetBased then
			if self.TeamBudget[team] - price < 0 then
				ok2spawn = false
			end
		end
		
		if ok2spawn then
			if self.BudgetBased then
				local count = tonumber(self.Data[faction.."-UnitCount"])
				if count == nil then
					count = 0
				end
				
				count = count + 1
			
				self.Data[faction.."-UnitCount"] = count
				
				local totalprice = tonumber(self.Data[faction.."-TotalPrice"])
				if totalprice == nil then
					totalprice = 0
				end
				
				totalprice = totalprice + price
			
				self.Data[faction.."-TotalPrice"] = totalprice
				
				--print (faction.." - "..totalprice)
			end
		
			actor = CF_MakeActor(CF_ActPresets[faction][r1] , CF_ActClasses[faction][r1], CF_ActModules[faction][r1]);
			
			if actor ~= nil then
				self.TeamBudget[team] = self.TeamBudget[team] - price
				--print (tostring(team).." "..self.TeamBudget[team])

				if class == "AHuman" then
					if r2 ~= nil then
						item = CF_MakeItem(CF_ItmPresets[faction][r2] , CF_ItmClasses[faction][r2], CF_ItmModules[faction][r2]);
						
						if item ~= nil then
							actor:AddInventoryItem(item);
						end
					end
				end
				
				actor.AIMode = aimode;
				actor.Team = team;
				
				if pos ~= nil then
					actor.Pos = pos;
					MovableMan:AddActor(actor);
				else
					return actor
				end
			end
		end
	end
	
	return nil
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function WeaponTest:SpawnActors()
	print ("Spawn actors")
	self.TeamBudget = {}

	-- Set record ID
	local f1 = self.StrFaction1
	local f2 = self.StrFaction2
	
	if f1 < f2 then
		self.RecordID = f1 .."-"..f2
	else
		self.RecordID = f2 .."-"..f1
	end
	
	local tp = ""
	
	if self.BudgetBased then
		tp = "Budget"
	else
		tp = "Count"
	end
	
	self.RecordID = tp.."-"..self.RecordID
	
	self.Round = tonumber(self.Data[self.RecordID.."-Round"])
	if self.Round == nil then
		self.Round = 0
	end
	
	self.ResultFaction1 = self.StrFaction1
	self.ResultFaction2 = self.StrFaction2
	
	-- Select spawn areas
	local spawn = {}
	
	if self.Round % 2 == 0 then
		spawn[1] = self.Spawn[0]
		spawn[2] = self.Spawn[1]
	else
		spawn[1] = self.Spawn[1]
		spawn[2] = self.Spawn[0]
	end
	
	for t = 1, 2 do
		self.TeamBudget[t] = self.Budget
		
		local f
		if t == 1 then
			f = self.StrFaction1
		end
		
		if t == 2 then
			f = self.StrFaction2
		end
		
		local count = 8
		
		if not self.BudgetBased then
			count = self.SpawnAmount
		else
			count = 20
		end
		
		for i = 1, count do
			local actor = self:SpawnRandomInfantry(t, nil, f, Actor.AIMODE_SENTRY)
			if actor ~= nil then
				actor.Pos = Vector(spawn[t]:GetRandomPoint().X, spawn[t]:GetCenterPoint().Y)
				MovableMan:AddActor(actor);
				
				--print(f.." - "..i.." - "..self.TeamBudget[t])
			end
		end

		if self.BudgetBased then
			local totalrounds = tonumber(self.Data[f.."-TotalRounds"])
			if totalrounds == nil then
				totalrounds = 0
			end
			totalrounds = totalrounds + 1
			self.Data[f.."-TotalRounds"] = totalrounds
		end
	end
	
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function WeaponTest:ProcessUI()
	--print (self.StrFaction2)

	local act = self:GetControlledActor(self.PlayerTeam);
	
	-- Process delivery controls
	if MovableMan:IsActor(act) and act.PresetName == "Controller" then
		local cont = act:GetController()
		
		-- Switch to prev preset
		if cont:IsState(Controller.PRESS_UP) then
			self.SelectedValue = self.SelectedValue - 1
			if self.SelectedValue < 1 then
				self.SelectedValue = 6
			end
		end

		-- Switch to next preset
		if cont:IsState(Controller.PRESS_DOWN) then
			self.SelectedValue = self.SelectedValue + 1
			if self.SelectedValue > 6 then
				self.SelectedValue = 1
			end
		end

		if cont:IsState(Controller.PRESS_LEFT) or cont:IsState(Controller.PRESS_RIGHT) then
			if self.PrimaryFaction == nil then
				if self.SelectedValue == 1 or self.SelectedValue == 3 or self.SelectedValue == 4 then
					if cont:IsState(Controller.PRESS_LEFT) then
						self.Faction1 = self.Faction1 - 1
						if self.Faction1 == 0 then
							self.Faction1 = #self.PlayableFactions
						end
					end
				
					if cont:IsState(Controller.PRESS_RIGHT) then
						self.Faction1 = self.Faction1 + 1
						if self.Faction1 == #self.PlayableFactions + 1 then
							self.Faction1 = 1
						end
					end
				end
			end

			if self.TestedAgainstFactions == nil then
				if self.SelectedValue == 2 then
					if cont:IsState(Controller.PRESS_LEFT) then
						self.Faction2 = self.Faction2 - 1
						if self.Faction2 == 0 then
							self.Faction2 = #self.PlayableFactions
						end
					end
				
					if cont:IsState(Controller.PRESS_RIGHT) then
						self.Faction2 = self.Faction2 + 1
						if self.Faction2 == #self.PlayableFactions + 1 then
							self.Faction2 = 1
						end
					end
				end
			end
			
			if self.SelectedValue == 5 then
				if cont:IsState(Controller.PRESS_LEFT) or cont:IsState(Controller.PRESS_RIGHT) then
					self.Randomize = not self.Randomize
				end
			end
			
			if self.SelectedValue == 6 then
				if cont:IsState(Controller.PRESS_LEFT) or cont:IsState(Controller.PRESS_RIGHT) then
					self.BudgetBased = not self.BudgetBased
				end
			end
		end

		if self.PrimaryFaction == nil then
			self.StrFaction1 = self.PlayableFactions[ self.Faction1 ]
		else
			self.StrFaction1 = self.PrimaryFaction
		end
		if self.TestedAgainstFactions == nil then
			self.StrFaction2 = self.PlayableFactions[ self.Faction2 ]
		end
		
		CF_DrawString(">", act.Pos + Vector(-60,self.SelectedValue * 10), 200, 200);

		CF_DrawString("Faction 1", act.Pos + Vector(-50 , 10), 200, 200);
		CF_DrawString(self.StrFaction1, act.Pos + Vector(10 , 10), 200, 200);
		if self.PrimaryFaction ~= nil then
			CF_DrawString("*", act.Pos + Vector(-70, 10), 250, 250);
		end
		if self.TestedAgainstFactions ~= nil then
			CF_DrawString("*", act.Pos + Vector(-70, 20), 250, 250);
		end
		
		CF_DrawString("Faction 2", act.Pos + Vector(-50 , 20), 200, 200);
		CF_DrawString(self.StrFaction2, act.Pos + Vector(10 , 20), 200, 200);

		
		CF_DrawString(self.StrFaction1.." stats", act.Pos + Vector(-50 , 30), 200, 200);

		CF_DrawString("Vanilla stats", act.Pos + Vector(-50 , 40), 200, 200);

		CF_DrawString("Randomize", act.Pos + Vector(-50 , 50), 200, 200);
		CF_DrawString(tostring(self.Randomize), act.Pos + Vector(10 , 50), 200, 200);

		CF_DrawString("Budget", act.Pos + Vector(-50 , 60), 200, 200);
		CF_DrawString(tostring(self.BudgetBased), act.Pos + Vector(10 , 60), 200, 200);

		local tp = ""
		
		if self.BudgetBased then
			tp = "Budget"
		else
			tp = "Count"
		end
		
		if self.SelectedValue == 3 then
			local id
			
			local f1 = self.StrFaction1
			local f2 = self.StrFaction2
			
			if f1 < f2 then
				id = f1 .."-"..f2
			else
				id = f2 .."-"..f1
			end
			
			id = tp.."-"..id
			
			local rounds = tonumber(self.Data[ id .."-Round"])
			if rounds == nil then
				rounds = 0
			end
			
			if rounds > 0 then
				local c1 = tonumber(self.Data[ id .."-"..f1])
				if c1 == nil then
					c1 = 0
				end

				local c2 = tonumber(self.Data[ id .."-"..f2])
				if c2 == nil then
					c2 = 0
				end
			
				CF_DrawString("Rounds: "..rounds, act.Pos + Vector(-50 , 80), 200, 200);
				CF_DrawString(f1..": "..c1, act.Pos + Vector(-50 , 90), 200, 200);
				CF_DrawString(f2..": "..c2, act.Pos + Vector(-50 , 100), 200, 200);
				if c2 > 0 then
					CF_DrawString("Ratio: "..string.format("%.2f", c1 / c2), act.Pos + Vector(-50 , 110), 200, 200);
				end
			else
				CF_DrawString("NO DATA", act.Pos + Vector(-50 , 80), 200, 200);
			end
			
			if self.BudgetBased then
				local totalrounds = tonumber(self.Data[f1.."-TotalRounds"])
				if totalrounds == nil then
					totalrounds = 0
				end				

				local count = tonumber(self.Data[f1.."-UnitCount"])
				if count == nil then
					count = 0
				end
				
				local totalprice = tonumber(self.Data[f1.."-TotalPrice"])
				if totalprice == nil then
					totalprice = 0
				end
				
				if totalrounds > 0 and count > 0 then
					CF_DrawString(string.format("Avg. squad: %.1f", count/totalrounds), act.Pos + Vector(-50 , 120), 200, 200);
					CF_DrawString("Avg. price: "..math.floor(totalprice/count), act.Pos + Vector(-50 , 130), 200, 200);
				end
			end
		end
		
		if self.SelectedValue == 4 then
			local vanilla = {"Coalition", "Dummy", "Ronin", "Techion", "Imperatus", "Browncoats"}
			local ps = 0
			
			for v = 1 , #vanilla do
				local id
				
				local f1 = self.StrFaction1
				local f2 = vanilla[v]

				if f1 ~= f2 then
					ps = ps + 1
					if f1 < f2 then
						id = f1 .."-"..f2
					else
						id = f2 .."-"..f1
					end
					
					id = tp.."-"..id					
					
					local rounds = tonumber(self.Data[ id .."-Round"])
					if rounds == nil then
						rounds = 0
					end
					
					CF_DrawString(vanilla[v], act.Pos + Vector(-50 , 70 + (ps * 10)), 200, 200);
					if rounds > 0 then
						local c1 = tonumber(self.Data[ id .."-"..f1])
						if c1 == nil then
							c1 = 0
						end

						local c2 = tonumber(self.Data[ id .."-"..f2])
						if c2 == nil then
							c2 = 0
						end
					
						if c2 > 0 then
							CF_DrawString(string.format("%.2f", c1 / c2), act.Pos + Vector(30 , 70 + (ps * 10)), 200, 200);
						end
					else
						CF_DrawString("NO DATA", act.Pos + Vector(30 , 70 + (ps * 10)), 200, 200);
					end
				end
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function WeaponTest:PauseActivity(pause)
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function WeaponTest:EndActivity()
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function WeaponTest:UpdateActivity()
	-- Just check for intialization flags in update loop to avoid unnecessary function calls during all the mission
	if self.IsInitialized == nil then
		self.IsInitialized = false
	end
	
	if not self.IsInitialized then
		--Init mission if we're still not
		self:StartActivity()
	end

	self:ClearObjectivePoints();
	
	self:ProcessUI()	

	-- Send all units to destination
	for actor in MovableMan.Actors do
		if actor.Team ~= 0 and actor.AIMode ~= Actor.AIMODE_GOTO then
			actor:ClearAIWaypoints()
			actor.AIMode = Actor.AIMODE_GOTO
			actor:AddAISceneWaypoint(self.Destination:GetCenterPoint())
		end
	end
	
	if self.GenericTimer:IsPastSimMS(self.EnemySpawnInterval * 1000) then
		self.GenericTimer:Reset();
		self.SpawnNextWave = true

		local surv1 = 0
		local surv2 = 0
		
		-- Save results
		for actor in MovableMan.Actors do
			if actor.Health > 0 and actor.Team ~= 0 then
				if actor.Team == 1 then
					surv1 = surv1 + 1
				end
			
				if actor.Team == 2 then
					surv2 = surv2 + 1
				end
			end
			
			if actor.Team ~= 0 then
				actor.ToDelete = true
			end
		end

		for item in MovableMan.Items do
			item.ToDelete = true
		end

		local tp = ""
		
		if self.BudgetBasedStart then
			tp = "Budget"
		else
			tp = "Count"
		end
		
		local total = tonumber(self.Data[tp.."-".."Total"])
		if total == nil then
			total = 0
		end
		total = total + 1
		self.Data[tp.."-".."Total"] = total
		
		---------------------------------
		self.Round = self.Round  + 1
		
		local count = tonumber(self.Data[self.RecordID.."-"..self.ResultFaction1])
		if count == nil then
			count = 0
		end
		
		count = count + surv1
		self.Data[self.RecordID.."-"..self.ResultFaction1] = count
		
		----------------------------------
		local count = tonumber(self.Data[self.RecordID.."-"..self.ResultFaction2])
		if count == nil then
			count = 0
		end
		
		count = count + surv2
		self.Data[self.RecordID.."-"..self.ResultFaction2] = count
		----------------------------------
		
		self.Data[self.RecordID.."-Round"]	= self.Round
		
		CF_WriteConfigFile(self.Data , self.ModuleName , "Stats.txt")
		
		-- Dump data to scv
		self:DumpStatsToCSV()
		
		self.LaunchCount = self.LaunchCount + 1
		if self.LaunchCount == self.RestartEvery and self.RestartEvery > 0 then
			ActivityMan:StartActivity("GAScripted", "Faction Test")
			return
		end
	end--]]--
	
	if self.SpawnNextWave then
		if self.Randomize then
			if self.PrimaryFaction == nil then
				self.Faction1 = math.random(#self.PlayableFactions)
				self.StrFaction1 = self.PlayableFactions[ self.Faction1 ]
			else
				self.StrFaction1 = self.PrimaryFaction
			end
			
			local ok = false
			
			while not ok do
				if self.TestedAgainstFactions == nil then
					self.Faction2 = math.random(#self.PlayableFactions)
					self.StrFaction2 = self.PlayableFactions[ self.Faction2 ]
				else
					self.StrFaction2 = self.TestedAgainstFactions[math.random(#self.TestedAgainstFactions)]
				end
				if self.StrFaction2 ~= self.StrFaction1 then
					ok = true
				end
			end
		end
		
		self.BudgetBasedStart = self.BudgetBased
	
		self:SpawnActors();
		self.SpawnNextWave = false
	end
	
	--self:DoBrainSelection()
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function WeaponTest:DoBrainSelection()
	if self.ActivityState ~= Activity.OVER then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);
			local brain = self:GetPlayerBrain(player);

			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				if team == self.PlayerTeam then
					self.PlayerBrainDead = true
				end

				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
					
					if team == self.PlayerTeam then
						self.PlayerBrainDead = false
					end
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end
		end
	end
end
-----------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------
function WeaponTest:ParseLine()

end
-----------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------
function WeaponTest:DumpStatsToCSV()
	local prefix = {}
	prefix[0] = "Budget"
	prefix[1] = "Count"

	local vanilla = {"Coalition", "Dummy", "Ronin","Browncoats", "Imperatus" , "Techion"}
	
	for p = 0 , 1 do
		io = require("io");

		local file = io.open("./WeaponTest.rte/"..prefix[p]..".csv" , "w");
		if p == 0 then
			file:write("Faction;Coalition;Dummy;Ronin;Browncoats;Imperatus;Techion;Avg power;Avg squad size;Avg unit price;\n");
		else
			file:write("Faction;Coalition;Dummy;Ronin;Browncoats;Imperatus;Techion;Avg power;\n");
		end

		local data = {}
	
		for fn = 1, #CF_Factions do
			if CF_FactionPlayable[CF_Factions[fn]] then
				local f1 = CF_Factions[fn]
				data[f1] = {}
				
				local totalpower = 0
				local divider = 0
				
				for e = 1, #vanilla do
					if f1 ~= f2 then
						local f2 = vanilla[e]
						local recid
						if f1 < f2 then
							recid = f1 .."-"..f2
						else
							recid = f2 .."-"..f1
						end

						local we = tonumber(self.Data[prefix[p].."-"..recid.."-"..f1])
						local enemies = tonumber(self.Data[prefix[p].."-"..recid.."-"..f2])
						local rounds = tonumber(self.Data[prefix[p].."-"..recid.."-Round"])
						
						if we ~= nil and enemies ~= nil and rounds ~= nil then
							if enemies > 0 then
								data[f1][e] = string.format("%.2f", we / enemies).." ( "..rounds.." )"
								
								totalpower = totalpower + (we / enemies)
								divider = divider + 1
							else
								data[f1][e] = "n\\a"
							end
						else
							data[f1][e] = "n\\a"
						end
					else
						data[f1][e] = "n\\a"
					end
				end
				
				if totalpower > 0 and divider > 0 then
					data[f1][#vanilla + 1] = string.format("%.2f", totalpower / divider)
					data[f1][#vanilla + 1] = string.gsub(data[f1][#vanilla + 1], "%.", ",")
				else
					data[f1][#vanilla + 1] = "n\\a"
				end
				
				if p == 0 then
					local rnds = tonumber(self.Data[f1.."-TotalRounds"])
					local price = tonumber(self.Data[f1.."-TotalPrice"])
					local count = tonumber(self.Data[f1.."-UnitCount"])
				
					if rnds ~= nil and price ~= nil and count ~= nil then
						if rnds > 0 then
							data[f1][#vanilla + 2] = string.format("%.1f", count / rnds)
							data[f1][#vanilla + 2] = string.gsub(data[f1][#vanilla + 2], "%.", ",")
						else
							data[f1][#vanilla + 2] = "n\\a"
						end

						if count > 0 then
							data[f1][#vanilla + 3] = math.floor(price / count)
						else
							data[f1][#vanilla + 3] = "n\\a"
						end
					else
						data[f1][#vanilla + 2] = "n\\a"
						data[f1][#vanilla + 3] = "n\\a"
					end
				end
				
				--print (f1)
				
				-- Write line
				local ln = f1..";"
				for i = 1,  #data[f1] do
					ln = ln..data[f1][i]..";"
					--print (ln)
				end
				file:write(ln.."\n");
			end
		end
		
		file:close()
	end
end
-----------------------------------------------------------------------------------------
-- Thats all folks!!!
-----------------------------------------------------------------------------------------
